#include "types.h"
void init_i2c(void);
extern int i2cstat;
int i2c_wait(void);
void i2c_go(void);
int i2c_conreg(void);
int i2c_statreg(void);
bool i2c_send_start(void);
bool i2c_send_address(int addr, bool write);
bool i2c_send_data(int data);
void i2c_send_stop(void);
